/*-
 * based on src/usr.sbin/sysinstall/disks.c,v 1.154.2.2 2006/02/27 20:01:31 jkim
 *
 * Copyright (c) 1995
 *	Jordan Hubbard.  All rights reserved.
 *
 * Copyright (c) 2006 Alex Kozlov <spam@rm-rf.kiev.ua>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    verbatim and that no modifications are made prior to this
 *    point in the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JORDAN HUBBARD ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JORDAN HUBBARD OR HIS PETS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, LIFE OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: disks.c,v 1.28 2006/04/23 07:35:52 stalker Exp stalker $
 */

#include <sys/stat.h>
#include <ctype.h>
#include <inttypes.h>
#include <fcntl.h>
#include <libdisk.h>
#include "fdisk.h"

/* prototypes */
static void		to_last_chunk(void);
static void		record_chunks(Disk *d);
static void		print_chunks(Disk *d, int u);
static void		print_command_summary();
static u_char	*bootalloc(char *name, size_t *size);
static int		diskPartitionWrite(Disk *d);
static void		Commit(Disk *d);
static void		InstallMbr(Disk *d);
#ifdef PC98
static void
getBootMgr(char *dname, u_char **bootipl, size_t *bootipl_size,
		u_char **bootmenu, size_t *bootmenu_size);
#else
static void
getBootMgr(char *dname, u_char **bootCode, size_t *bootCodeSize);
#endif

/* globals */
int				BootMgr;
Boolean			Disk_Partitioned;
static struct	chunk *chunk_info[CHUNK_INFO_ENTRIES];
static int		current_chunk;

#if defined(__i386__) || defined(__amd64__)
#ifdef PC98
/* IPL type menu */
DMenu MenuIPLType = {
	DMENU_NORMAL_TYPE | DMENU_SELECTION_RETURNS,
	"overwrite me",		/* will be disk specific label */
	"If you want a FreeBSD Boot Manager, select \"BootMgr\".  If you would\n"
	"prefer your Boot Manager to remain untouched then select \"None\".\n\n",
	"Press F1 to read about drive setup",
	"drives",
	{	{ "BootMgr",	"Install the FreeBSD Boot Manager",
	dmenuRadioCheck, dmenuSetValue, NULL, &BootMgr },
		{ "None",		"Leave the IPL untouched",
	dmenuRadioCheck, dmenuSetValue, NULL, &BootMgr, '(', '*', ')', 1 },
		{ NULL } },
};
#else
/* MBR type menu */
DMenu MenuMBRType = {
	DMENU_NORMAL_TYPE | DMENU_SELECTION_RETURNS,
	"overwrite me",		/* will be disk specific label */
	"FreeBSD comes with a boot selector that allows you to easily\n"
	"select between FreeBSD and any other operating systems on your machine\n"
	"at boot time.  If you have more than one drive and want to boot\n"
	"from the second one, the boot selector will also make it possible\n"
	"to do so (limitations in the PC BIOS usually prevent this otherwise).\n"
	"If you do not want a boot selector, or wish to replace an existing\n"
	"one, select \"standard\".  If you would prefer your Master Boot\n"
	"Record to remain untouched then select \"None\".\n\n"
	"  NOTE:  PC-DOS users will almost certainly require \"None\"!",
	"Press F1 to read about drive setup",
	"drives",
	{	{ "BootMgr",	"Install the FreeBSD Boot Manager",
	dmenuRadioCheck, dmenuSetValue, NULL, &BootMgr },
		{ "Standard",	"Install a standard MBR (no boot manager)",
	dmenuRadioCheck, dmenuSetValue, NULL, &BootMgr, '(', '*', ')', 1 },
		{ "None",		"Leave the Master Boot Record untouched",
	dmenuRadioCheck, dmenuSetValue, NULL, &BootMgr, '(', '*', ')', 2 },
		{ NULL } },
};
#endif /* PC98 */
#endif /* __i386__ */

/* functions */
static void
InstallMbr(Disk *d)
{
#ifdef PC98
	u_char *bootipl;
	size_t bootipl_size;
	u_char *bootmenu;
	size_t bootmenu_size;
#else
	u_char *mbrContents;
	size_t mbrSize;
#endif

#ifdef PC98
	/*
	 * Don't trash the IPL if the first (and therefore only) chunk
	 * is marked for a truly dedicated disk (i.e., the disklabel
	 * starts at sector 0), even in cases where the user has
	 * requested a FreeBSD Boot Manager -- both would be fatal in
	 * this case.
	 */
	/*
	 * Don't offer to update the IPL on this disk if the first
	 * "real" chunk looks like a FreeBSD "all disk" partition,
	 * or the disk is entirely FreeBSD.
	 */
	if ((d->chunks->part->type != freebsd) ||
			(d->chunks->part->offset > 1)) {
			getBootMgr(d->name, &bootipl, &bootipl_size,
				&bootmenu, &bootmenu_size);
	} else {
			bootipl = NULL;
			bootipl_size = 0;
			bootmenu = NULL;
			bootmenu_size = 0;
	}
	Set_Boot_Mgr(d, bootipl, bootipl_size, bootmenu, bootmenu_size);
#else
	/*
	 * Don't trash the MBR if the first (and therefore only) chunk
	 * is marked for a truly dedicated disk (i.e., the disklabel
	 * starts at sector 0), even in cases where the user has
	 * requested booteasy or a "standard" MBR -- both would be
	 * fatal in this case.
	 */
	/*
	 * Don't offer to update the MBR on this disk if the first
	 * "real" chunk looks like a FreeBSD "all disk" partition,
	 * or the disk is entirely FreeBSD.
	 */
	if ((d->chunks->part->type != freebsd) ||
			(d->chunks->part->offset > 1)) {
			getBootMgr(d->name, &mbrContents, &mbrSize);
	} else {
			mbrContents = NULL;
			mbrSize = 0;
	}
	Set_Boot_Mgr(d, mbrContents, mbrSize);
#endif
}

#ifdef PC98
static void
getBootMgr(char *dname, u_char **bootipl, size_t *bootipl_size,
	u_char **bootmenu, size_t *bootmenu_size)
{
	static u_char *boot0;
	static size_t boot0_size;
	static u_char *boot05;
	static size_t boot05_size;

	char str[80];
	int i = 0;

	sprintf(str, "Install Boot Manager for drive %s?", dname);
	MenuIPLType.title = str;
	i = dmenuOpenSimple(&MenuIPLType, FALSE);
 
	if (i) {
	switch (BootMgr) {
	case 0:
		if (!boot0)
				boot0 = bootalloc("boot0", &boot0_size);

		*bootipl = boot0;
		*bootipl_size = boot0_size;
		if (!boot05)
				boot05 = bootalloc("boot0.5", &boot05_size);

		*bootmenu = boot05;
		*bootmenu_size = boot05_size;
		return;
	case 1:
	default:
		break;
		}
	}
	*bootipl = NULL;
	*bootipl_size = 0;
	*bootmenu = NULL;
	*bootmenu_size = 0;
}
#else
static void
getBootMgr(char *dname, u_char **bootCode, size_t *bootCodeSize)
{
#if defined(__i386__) || defined(__amd64__)	/* only meaningful on x86 */
	static u_char *mbr, *boot0;
	static size_t mbr_size, boot0_size;
	char str[80];
	int i = 0;

	/* Figure out what kind of MBR the user wants */
	sprintf(str, "Install Boot Manager for drive %s?", dname);
	MenuMBRType.title = str;
	i = dmenuOpenSimple(&MenuMBRType, FALSE);

	if (i) {
		switch (BootMgr) {
		case 0:
			if (!boot0)
					boot0 = bootalloc("boot0", &boot0_size);

			*bootCode = boot0;
			*bootCodeSize = boot0_size;
			return;
		case 1:
			if (!mbr)
					mbr = bootalloc("mbr", &mbr_size);

			*bootCode = mbr;
			*bootCodeSize = mbr_size;
			return;
		case 2:
		default:
			break;
		}
	}
#endif
	*bootCode = NULL;
	*bootCodeSize = 0;
}
#endif

static u_char *
bootalloc(char *name, size_t *size)
{
	char buf[FILENAME_MAX];
	struct stat sb;

	snprintf(buf, sizeof(buf), "/boot/%s", name);
	if (stat(buf, &sb) != -1) {
		int fd;

		fd = open(buf, O_RDONLY);
		if (fd != -1) {
			u_char *cp;

			cp = malloc(sb.st_size);
			if (read(fd, cp, sb.st_size) != sb.st_size) {
				free(cp);
				close(fd);
				msgDebug("bootalloc: couldn't read %ld bytes from %s\n",
							(long)sb.st_size, buf);
				return NULL;
			}
			close(fd);
			if (size != NULL)
				*size = sb.st_size;

			return cp;
		}
		msgDebug("bootalloc: couldn't open %s\n", buf);
	} else
		msgDebug("bootalloc: can't stat %s\n", buf);

	return NULL;
}

static int
diskPartitionWrite(Disk *d)
{
	static u_char *boot1;
#if defined(__i386__) || defined(__amd64__)
	static u_char *boot2;
#endif
#if defined(__i386__) || defined(__amd64__)
	if (!boot1) boot1 = bootalloc("boot1", NULL);
	if (!boot2) boot2 = bootalloc("boot2", NULL);
	Set_Boot_Blocks(d, boot1, boot2);
#elif !defined(__ia64__)
	if (!boot1) boot1 = bootalloc("boot1", NULL);
	Set_Boot_Blocks(d, boot1, NULL);
#endif

	if (!Fake && Write_Disk(d))
		return FALSE;

	/* Now it's not "yes", but "written" */
	Disk_Partitioned = WRITTEN;
	return TRUE;
}

static void
Commit(Disk *d)
{
	char *p;

	p = CheckRules(d);
	if (p) {
		use_helpline("Press F1 to read more about disk slices.");
		use_helpfile(FindHelpFile("partition"));
		dialog_mesgbox("Disk slicing warning", p, -1, -1);
		free(p);

		/* don't write bad PT */
		return;
	}

	if (diskPartitionWrite(d) != TRUE)
		msgConfirm("Disk partition write on %s returned an error status!", d->name);
	else
		msgNotify("Wrote FDISK partition information out successfully.");
}

void
editor(char *diskname)
{
	Disk *d;
	WINDOW *w;
	int rv, key = 0;
	int i;
	int size_unit;
	Boolean chunking;
	char *msg = NULL;
	
	d = Open_Disk(diskname);
	
	w = savescr();

	size_unit = UNIT_BLOCKS;
	Disk_Partitioned = NO;
	chunking = TRUE;
	keypad(stdscr, TRUE);

	dialog_clear_norefresh(); clear();
	current_chunk = 0;

	/* Set up the chunk array */
	record_chunks(d);

	while (chunking) {
		char *val, geometry[80];

		/* Now print our overall state */
		if (d)
			print_chunks(d, size_unit);

		print_command_summary();
		
		if (msg) {
			msgHighlight(StatusLine, 0, msg);
			beep();
			msg = NULL;
		} else {
			/* clear Debug and StatusLine */
			move(StatusLine-1, 0);
			clrtoeol();
			move(StatusLine, 0);
			clrtoeol();
		}

		/* Get command character */
		key = getch();
		switch (toupper(key)) {
		case '\014':	/* ^L (redraw) */
			clear();
			msg = NULL;
			break;

		case '\020':	/* ^P */
		case KEY_UP:
		case '-':
			if (current_chunk != 0)
				current_chunk--;
			else
				to_last_chunk();
			break;

		case '\016':	/* ^N */
		case KEY_DOWN:
		case '+':
		case '\r':
		case '\n':
			if (current_chunk < CHUNK_INFO_ENTRIES - 1 &&
					current_chunk < MAX_CHUNK_DISPLAY &&
					chunk_info[current_chunk + 1]) {
				current_chunk++;
			} else
				current_chunk = 0;
			break;

		case KEY_HOME:
			current_chunk = 0;
			break;

		case KEY_END:
			to_last_chunk();
			break;

		case KEY_F(1):
		case '?':
			DisplayHelp("slice");
			clear();
			break;

		case 'A':
		case 'F':	/* Undocumented magic Dangerously Dedicated mode */
#if !defined(__i386__) && !defined(__amd64__)
			rv = 1;
#else				/* The rest is only relevant on x86 */
			if (toupper(key) == 'A')
				rv = 0;
			else {
				rv = msgYesNo("Do you want to do this with a true partition entry\n"
					"so as to remain cooperative with any future possible\n"
					"operating systems on the drive(s)?\n"
					"(See also the section about ``dangerously dedicated''\n"
					"disks in the FreeBSD FAQ.)");
			}
			if (rv == -1)
				rv = 0;
#endif
			All_FreeBSD(d, rv);
			Disk_Partitioned = YES;
			record_chunks(d);
			clear();
			break;
 
		case 'C':
			if (chunk_info[current_chunk]->type != unused) {
				msg = "Slice in use, delete it first or move to an unused one.";
			} else if (chunk_info[current_chunk]->offset == 0 &&
				/* Never allow write to the first track/cylinder */
				chunk_info[current_chunk]->size == RESERVED_SIZE) {
				msg = "Reserved slice.";
			} else {
				char *val, tmp[20], name[16], *cp;
				daddr_t size;
				int subtype;
				chunk_e partitiontype;
#ifdef PC98
				snprintf(name, sizeof (name), "%s", "FreeBSD");
				val = msgGetInput(name,
					"Please specify the name for new FreeBSD slice.");
				if (val)
					strncpy(name, val, sizeof (name));
#else
				name[0] = '\0';
#endif
				snprintf(tmp, 20, "%jd", 
						(intmax_t)chunk_info[current_chunk]->size);
				val = msgGetInput(tmp, 
				"Please specify the size for new FreeBSD slice in blocks\n"
					"or append a trailing `M' for megabytes (e.g. 20M).");
				if (val && (size = strtoimax(val, &cp, 0)) > 0) {
					if (*cp && toupper(*cp) == 'M')
						size *= ONE_MEG;
					else if (*cp && toupper(*cp) == 'G')
						size *= ONE_GIG;

					sprintf(tmp, "%d", SUBTYPE_FREEBSD);
					val = msgGetInput(tmp, "Enter type of partition to create:\n\n"
					"Pressing Enter will choose the default, a native FreeBSD\n"
					"slice (type %u)."
					OTHER_SLICE_VALUES
					NON_FREEBSD_NOTE, SUBTYPE_FREEBSD);
					if (val && (subtype = strtol(val, NULL, 0)) > 0) {
						if (subtype == SUBTYPE_FREEBSD)
							partitiontype = freebsd;
						else if (subtype == SUBTYPE_FAT)
							partitiontype = fat;
						else if (subtype == SUBTYPE_EFI)
							partitiontype = efi;
						else
#ifdef PC98
							partitiontype = pc98;
#else
							partitiontype = mbr;
#endif
						Create_Chunk(d, chunk_info[current_chunk]->offset,
							size, partitiontype, subtype,
					 		 (chunk_info[current_chunk]->flags & CHUNK_ALIGN), name);
						Disk_Partitioned = YES;
						record_chunks(d);
					}
				}
			clear();
			}
			break;

		case KEY_DC:
		case 'D':
			if (chunk_info[current_chunk]->type == unused) {
				msg = "Slice is already unused!";
			} else {
				Delete_Chunk(d, chunk_info[current_chunk]);
				Disk_Partitioned = YES;
				record_chunks(d);
			}
		 	break;
	 
		case 'T':
			if (chunk_info[current_chunk]->type == unused) {
				msg = "Slice is currently unused (use create instead)";
			} else {
				char *val, tmp[20];
				int subtype;
				chunk_e partitiontype;

				sprintf(tmp, "%d", chunk_info[current_chunk]->subtype);
				val = msgGetInput(tmp, "New partition type:\n\n"
				"Pressing Enter will use the current type. To choose a native\n"
					"FreeBSD slice enter %u."
				OTHER_SLICE_VALUES
				NON_FREEBSD_NOTE, SUBTYPE_FREEBSD);

				if (val && (subtype = strtol(val, NULL, 0)) > 0) {
					if (subtype == SUBTYPE_FREEBSD)
						partitiontype = freebsd;
					else if (subtype == SUBTYPE_FAT)
						partitiontype = fat;
					else if (subtype == SUBTYPE_EFI)
						partitiontype = efi;
					else
#ifdef PC98
					partitiontype = pc98;
#else
					partitiontype = mbr;
#endif
					chunk_info[current_chunk]->type = partitiontype;
					chunk_info[current_chunk]->subtype = subtype;
				}
			}
			break;
	 
		case 'G':
			snprintf(geometry, 80, "%lu/%lu/%lu", d->bios_cyl, d->bios_hd, d->bios_sect);
			val = msgGetInput(geometry, "Please specify the new geometry in cyl/hd/sect format.\n"
			"Don't forget to use the two slash (/) separator characters!\n"
			"It's not possible to parse the field without them.");
			if (val) {
				long nc, nh, ns;
				nc = strtol(val, &val, 0);
				nh = strtol(val + 1, &val, 0);
				ns = strtol(val + 1, 0, 0);
				Set_Bios_Geom(d, nc, nh, ns);
			}
			clear();
			break;
	
		case 'S':
			/* Clear active states so we won't have two */
			for (i = 0; (chunk_info[i] != NULL) &&
							(i < CHUNK_INFO_ENTRIES); i++) {
				chunk_info[i]->flags &= !CHUNK_ACTIVE;
			}
		 	/* Set Bootable */
		 	chunk_info[current_chunk]->flags |= CHUNK_ACTIVE;
	 		break;
	
		case 'U':
			if (!msgYesNo("Are you SURE you want to Undo everything?")) {
				char cp[BUFSIZ];

				sstrncpy(cp, d->name, sizeof(cp));
				Free_Disk(d);
				d = Open_Disk(cp);

				if (!d)
				 	msgConfirm("Can't reopen disk %s!\
							 	Internal state is probably corrupted", cp);

				Disk_Partitioned = NO;
				if (d)
					record_chunks(d);
		 	}
		 	clear();
		 	break;

		case 'W':
			Disk_Partitioned = YES;
			Commit(d);
		 	clear();
			break;

		case 'B':
			Disk_Partitioned = YES;
			InstallMbr(d);
			clear();
			break;

		case '\033':	/* ESC */
		case 'Q':
			chunking = FALSE;

			if (Disk_Partitioned == YES && Disk_Partitioned != WRITTEN) {
				if (!msgNoYes("Partition table or Mbr on (%s) has been changed\n"
					"Are you sure you want to write this changes?", d->name)) {
					Commit(d);
				}
			}
		 	break;

		case 'Z':
			size_unit = (size_unit + 1) % UNIT_SIZE;
		 	break;

		default:
			beep();
			msg = "Type F1 or ? for help";
			break;
		}
	}
	restorescr(w);
}

static void 
to_last_chunk(void)
{
	while (current_chunk < CHUNK_INFO_ENTRIES - 1 &&
			current_chunk < MAX_CHUNK_DISPLAY &&
				chunk_info[current_chunk + 1]) {
		current_chunk++;
	}
}

static void
record_chunks(Disk *d)
{
	struct chunk *c1 = NULL;
	int i = 0;
	daddr_t last_free = 0;

	if (!d->chunks)
		msgFatal("No chunk list found for %s!", d->name);


	for (c1 = d->chunks->part; i < CHUNK_INFO_ENTRIES && 
					i < MAX_CHUNK_DISPLAY &&
					c1; c1 = c1->next) {
		if (c1->type == unused && c1->size > last_free) {
	 		last_free = c1->size;
	 		current_chunk = i;
		}
		chunk_info[i++] = c1;
	}
	chunk_info[i] = NULL;
	if (current_chunk >= i)
		current_chunk = i - 1;
}

static void
print_chunks(Disk *d, int u)
{
	int row;
	int i;
	daddr_t sz;
	char *szstr;
	
	Boolean last_chunk = FALSE;

	szstr = (u == UNIT_GIG ? "GB" : (u == UNIT_MEG ? "MB" :
	(u == UNIT_KILO ? "KB" : "ST")));

	if (d->bios_cyl >= BIOS_CYL || d->bios_hd > BIOS_HD ||
					d->bios_sect > BIOS_SECT) {
		msgConfirm("WARNING:  A geometry of %lu/%lu/%lu for %s is incorrect.  Using\n"
			"a more likely geometry.  If this geometry is incorrect or you\n"
			"are unsure as to whether or not it's correct, please consult\n"
			"the Hardware Guide in the Documentation submenu or use the\n"
			"(G)eometry command to change it now.\n\n"
			"Remember: you need to enter whatever your BIOS thinks the\n"
			"geometry is!  For IDE, it's what you were told in the BIOS\n"
			"setup. For SCSI, it's the translation mode your controller is\n"
			"using.  Do NOT use a ``physical geometry''.",
			d->bios_cyl, d->bios_hd, d->bios_sect, d->name);
		Sanitize_Bios_Geom(d);
		dialog_clear_norefresh(); clear();
	}
	attrset(A_NORMAL);
	mvprintw(0, 0, "Disk name:\t");
	msgHighlight(0, 16, "%s", d->name);
	msgHighlight(0, 55, "FDISK Partition Editor");
	mvprintw(1, 0,
		"DISK Geometry:\t%lu cyls/%lu heads/%lu sectors = %jd sectors (%jdMB)",
		d->bios_cyl, d->bios_hd, d->bios_sect,
		(intmax_t)d->bios_cyl * d->bios_hd * d->bios_sect,
		(intmax_t)d->bios_cyl * d->bios_hd * d->bios_sect / (1024/512) / 1024);
	mvprintw(3, 0, "%6s %10s(%s) %10s %8s %6s %10s %8s %8s",
		"Offset", "Size", szstr, "End", "Name", "PType", "Desc",
		"Subtype", "Flags");
	for (i = 0, row = CHUNK_START_ROW;
			i < CHUNK_INFO_ENTRIES &&
			i < MAX_CHUNK_DISPLAY;
			i++, row++) {
		if(!last_chunk && chunk_info[i]) {
			switch(u) {
			default:	/* fall thru */
			case UNIT_BLOCKS:
				sz = chunk_info[i]->size;
				break;
			case UNIT_KILO:
				sz = chunk_info[i]->size / (1024/512);
				break;
			case UNIT_MEG:
				sz = chunk_info[i]->size / (1024/512) / 1024;
				break;
			case UNIT_GIG:
				sz = chunk_info[i]->size / (1024/512) / 1024 / 1024;
			 	break;
			}
			mvprintw(row, 0, "%10jd %10jd %10jd %8s %6d %10s %8d\t%-6s",
			 (intmax_t)chunk_info[i]->offset, (intmax_t)sz,
			 (intmax_t)chunk_info[i]->end, chunk_info[i]->name,
			 chunk_info[i]->type, 
			 slice_type_name(chunk_info[i]->type, chunk_info[i]->subtype),
			 chunk_info[i]->subtype, ShowChunkFlags(chunk_info[i]));
			if (i == current_chunk)
				mvchgat(row, 0, 78, A_REVERSE, 0, NULL);

		} else {
			last_chunk = TRUE;
			move(row, 0);
			clrtoeol();
		}
	}
}

static void
print_command_summary()
{
	mvprintw(StatusLine - 7, 0, "The following commands are supported (in upper or lower case):");
	mvprintw(StatusLine - 6, 0, "A = Use Entire Disk   G = set Drive Geometry   C = Create Slice   F = `DD' mode");
	mvprintw(StatusLine - 5, 0, "D = Delete Slice      Z = Toggle Size Units    S = Set Bootable   B = Inst. MBR");
	mvprintw(StatusLine - 4, 0, "T = Change Type       U = Undo All Changes     W = Write Changes  Q = Finish");
	mvprintw(StatusLine - 2, 0, "Use F1 or ? to get more help, arrow keys to select, Q or Esc to exit.");
	move(0, 0);
}
