/*
 * based on src/usr.sbin/sysinstall/dmenu.c,v 1.45.10.1 2006/03/07 16:06:40 jhb
 *
 * Copyright (c) 1995
 *	Jordan Hubbard.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    verbatim and that no modifications are made prior to this
 *    point in the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JORDAN HUBBARD ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JORDAN HUBBARD OR HIS PETS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, LIFE OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: dmenu.c,v 1.8 2006/04/22 19:19:40 stalker Exp $
 */

#include <unistd.h>
#include "fdisk.h"

int
dmenuRadioCheck(dialogMenuItem *item)
{
    return (*((unsigned int *)item->data) == item->aux);
}

int
dmenuSetValue(dialogMenuItem *tmp)
{
    *((unsigned int *)tmp->data) = tmp->aux;
    return DITEM_SUCCESS;
}

/* Traverse menu but give user no control over positioning */
Boolean
dmenuOpenSimple(DMenu *menu, Boolean buttons)
{
    int choice, scroll, curr, max;

    choice = scroll = curr = max = 0;
    return dmenuOpen(menu, &choice, &scroll, &curr, &max, buttons);
}

/* Traverse over an internal menu */
Boolean
dmenuOpen(DMenu *menu, int *choice, int *scroll, int *curr, int *max, Boolean buttons)
{
    int n, rval = 0;
    dialogMenuItem *items;

    items = menu->items;
    if (buttons) {
		items += 2;
	}
    /* Count up all the items */
    for (n = 0; items[n].title; n++);

    while (1) {
		WINDOW *w = savescr();

	/* Any helpful hints, put 'em up! */
	use_helpline(menu->helpline);
	use_helpfile(FindHelpFile(menu->helpfile));
	dialog_clear();
	/* Pop up that dialog! */
		if (menu->type & DMENU_NORMAL_TYPE) {
		    rval = dialog_menu((u_char *)menu->title, (u_char *)menu->prompt,
			-1, -1, menu_height(menu, n), -n, items,
			(char *)(uintptr_t)buttons, choice, scroll);

		} else if (menu->type & DMENU_RADIO_TYPE) {
		    rval = dialog_radiolist((u_char *)menu->title,
			(u_char *)menu->prompt, -1, -1, menu_height(menu, n), -n,
			items, (char *)(uintptr_t)buttons);

		} else if (menu->type & DMENU_CHECKLIST_TYPE) {
		    rval = dialog_checklist((u_char *)menu->title,
			(u_char *)menu->prompt, -1, -1, menu_height(menu, n), -n,
			items, (char *)(uintptr_t)buttons);
		} else {
		    msgFatal("Menu: `%s' is of an unknown type\n", menu->title);
		}

		if (rval) {
		    restorescr(w);
		    return FALSE;
		} else if (menu->type & DMENU_SELECTION_RETURNS) {
		    restorescr(w);
		    return TRUE;
		}
    }
}

int
menu_height(DMenu *menu, int n)
{
    int max;
    char *t;

    max = MAX_MENU;
    if (StatusLine > 24)
	max += StatusLine - 24;
    for (t = menu->prompt; *t; t++) {
	if (*t == '\n')
	    --max;
    }
    return n > max ? max : n;
}
