/*-
 * Copyright (c) 2006 Alex Kozlov <spam@rm-rf.kiev.ua>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    verbatim and that no modifications are made prior to this
 *    point in the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JORDAN HUBBARD ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JORDAN HUBBARD OR HIS PETS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, LIFE OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: fdisk.h,v 1.15 2006/04/23 07:35:23 stalker Exp stalker $
 */ 

#ifndef _FDISK_H_INCLUDE
#define _FDISK_H_INCLUDE

#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dialog.h>

/** Types **/
typedef int Boolean;
typedef struct disk Disk;

/** help **/
#ifndef BASEDIR
#define BASEDIR "/usr/local"
#endif

#ifndef DOCDIR
#define DOCDIR "share/doc/sfdisk"
#endif

#ifndef DOCS
#define DOCS BASEDIR"/"DOCDIR
#endif

/** dmenu.c **/
/* Bitfields for menu options */
#define DMENU_NORMAL_TYPE		0x1	/* Normal dialog menu           */
#define DMENU_RADIO_TYPE		0x2	/* Radio dialog menu            */
#define DMENU_CHECKLIST_TYPE	0x4	/* Multiple choice menu         */
#define DMENU_SELECTION_RETURNS 0x8	/* Immediate return on item selection */

#define MAX_MENU		15

typedef struct _dmenu {
    int type;				/* What sort of menu we are	*/
    char *title;			/* Our title			*/
    char *prompt;			/* Our prompt			*/
    char *helpline;			/* Line of help at bottom	*/
    char *helpfile;			/* Help file for "F1"		*/
#if (__STDC_VERSION__ >= 199901L) || (__GNUC__ >= 3) 
    dialogMenuItem items[];		/* Array of menu items		*/
#elif __GNUC__
    dialogMenuItem items[0];		/* Array of menu items		*/
#else
#error "Create hack for C89 and K&R compilers."
#endif
} DMenu;

/** msg.c **/
enum _msgtype { MSG_FATAL, MSG_NOTIFY, MSG_CONFIRM, MSG_NOYES, MSG_YESNO, MSG_GETINPUT };

/** disks.c **/
enum size_units_t { UNIT_BLOCKS, UNIT_KILO, UNIT_MEG, UNIT_GIG, UNIT_SIZE };

#define DEV_NAME_MAX		128	/* The maximum length of a device name	*/

#define NO					0
#define YES					1
#define WRITTEN				111

/* Where we keep track of MBR chunks */
#define	CHUNK_INFO_ENTRIES	16

/* Where we start displaying chunk information on the screen */
#define CHUNK_START_ROW		5

#define MAX_CHUNK_DISPLAY	StatusLine - (CHUNK_START_ROW + 8)

/* One MB worth of blocks */
#define ONE_MEG				2048
#define ONE_GIG				(ONE_MEG * 1024)

#ifdef	PC98
#define	SUBTYPE_FREEBSD		50324
#define	SUBTYPE_FAT			37218
#else
#define	SUBTYPE_FREEBSD		165
#define	SUBTYPE_FAT			6
#endif
#define	SUBTYPE_EFI			239

/* First track/cylinder  size */
#ifdef PC98
#define RESERVED_SIZE d->bios_sect * d->bios_hd
#else
#define RESERVED_SIZE d->bios_sect
#endif

/* messages */
#ifdef PC98
#define	OTHER_SLICE_VALUES						\
	"Other popular values are 37218 for a\n"			\
	"DOS FAT partition.\n\n"
#else
#define	OTHER_SLICE_VALUES						\
	"Other popular values are 6 for a\n"				\
	"DOS FAT partition, 131 for a Linux ext2fs partition, or\n"	\
	"130 for a Linux swap partition.\n\n"
#endif
#define	NON_FREEBSD_NOTE						\
	"Note: If you choose a non-FreeBSD partition type, it will not\n" \
	"be formatted or otherwise prepared, it will simply reserve space\n" \
	"for you to use another tool, such as DOS format, to later format\n" \
	"and actually use the partition."

/* sync with check in Sanitize_Bios_Geom */
#ifdef PC98
#define BIOS_CYL	65536
#define BIOS_HD		255
#define BIOS_SECT	255
#else
#define BIOS_CYL	65536
#define BIOS_HD		255
#define BIOS_SECT	63
#endif

/** main.c **/
#define TTY_STATUS_LINE    23

/** prototypes **/
int			dmenuRadioCheck(dialogMenuItem *item);
int			dmenuSetValue(dialogMenuItem *tmp);
Boolean		dmenuOpenSimple(DMenu *menu, Boolean buttons);
Boolean		dmenuOpen(DMenu *menu, int *choice, int *scroll,
		   		int *curr, int *max, Boolean buttons);
int			menu_height(DMenu *menu, int n);

int			uni_msg(int mode, char *buf, char *fmt, ...);

int			DisplayHelp(char *file);
char		*FindHelpFile(char *file);
		
void		restorescr(WINDOW *w);
char		*sstrncpy(char *dst, const char *src, int size);

void		Shutdown(int status);

void		editor(char *diskname);
char		*check_arg(int argc, char *argv[]);

/** Handy  macro **/
#define file_readable(fname)	(Boolean) access(fname, F_OK) ? FALSE : TRUE
#define savescr(void)	(WINDOW*) dupwin(newscr)
#define msgGetInput(...) (char*)uni_msg(MSG_GETINPUT, __VA_ARGS__)
//#define msgDebug(...)	(void)	uni_msg(MSG_CONFIRM,	NULL, __VA_ARGS__)
#define msgDebug(...)	(void)	mvprintw(StatusLine-1, 0, __VA_ARGS__)
#define msgConfirm(...)	(void)	uni_msg(MSG_CONFIRM,	NULL, __VA_ARGS__)
#define msgNotify(...)	(void)	uni_msg(MSG_NOTIFY,	NULL, __VA_ARGS__)
#define msgFatal(...)	(void)	uni_msg(MSG_FATAL,	NULL, __VA_ARGS__)
#define msgNoYes(...)			uni_msg(MSG_NOYES,	NULL, __VA_ARGS__)
#define msgYesNo(...)			uni_msg(MSG_YESNO,	NULL, __VA_ARGS__)

#define msgHighlight(...)	{	attrset(A_REVERSE);\
								mvprintw(__VA_ARGS__);\
								attrset(A_NORMAL);\
}

/** externs **/
extern int		StatusLine;
extern Boolean	Fake;
extern Boolean	Debug;

#endif
/* _FDISK_H_INCLUDE */
