/*-
 * Copyright (c) 2006 Alex Kozlov <spam@rm-rf.kiev.ua>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    verbatim and that no modifications are made prior to this
 *    point in the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JORDAN HUBBARD ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JORDAN HUBBARD OR HIS PETS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, LIFE OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: main.c,v 1.6 2006/04/22 21:28:13 stalker Exp $
 */

#include <sys/ioctl.h>
#include <sys/consio.h>
#include <sys/errno.h>
#include <signal.h>
#include "fdisk.h"


int 	StatusLine;
Boolean Fake;
Boolean Debug;

static void
handle_intr(int sig)
{
	Shutdown(-1);
}

static char*
Initialize(int argc, char *argv[])
{
	sigset_t signalset;
	struct ttysize ts;
	char *diskname;

	Fake			 = FALSE;
	Debug			 = FALSE;

	signal(SIGINT, SIG_IGN);

	diskname = check_arg(argc, argv);

	if (ioctl(0, TIOCGSIZE, &ts) == -1) {
       msgDebug("Unable to get terminal size - errno %d\n", errno);
       ts.ts_lines = 0;
	}
	StatusLine = ts.ts_lines ? ts.ts_lines - 1: TTY_STATUS_LINE;

    init_dialog();

    signal(SIGINT, handle_intr);

    sigemptyset(&signalset);
    sigaddset(&signalset, SIGINT);
    sigprocmask(SIG_UNBLOCK, &signalset, NULL);

	return diskname;
}

void 
Shutdown(int status)
{
	/* Shut down the dialog & curses */
	end_dialog();
	endwin();

	exit(status);
}

int 
main(int argc, char *argv[])
{
	editor(Initialize(argc, argv));
	
	Shutdown(0);

	return (0);
}
