/*-
 * Based on src/usr.sbin/sysinstall/misc.c,v 1.43 2003/01/06 17:11:46 obrien
 *
 * Copyright (c) 1995
 *	Jordan Hubbard.  All rights reserved.
 *
 * Copyright (c) 2006 Alex Kozlov <spam@rm-rf.kiev.ua>
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    verbatim and that no modifications are made prior to this
 *    point in the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JORDAN HUBBARD ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JORDAN HUBBARD OR HIS PETS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, LIFE OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * 
 * $Id: misc.c,v 1.12 2006/04/23 07:36:12 stalker Exp stalker $
 */

#include <sys/stat.h>
#include <libdisk.h>
#include "fdisk.h"

static void
usage(void)
{
	fprintf(stderr, "usage: fdisk [-n] [-d] $disk\n");
	exit(2);
}

static char *
check_disk(char *diskname)
{
	char **cp;
	int i;
	Boolean valid;

	struct stat sb;
	char buf[20];

	if (strncmp("/dev/", diskname, 5) == 0)
			diskname+=5;

	valid = FALSE;
	cp = Disk_Names();
	for(i=0; cp[i]; i++) {
		if(strncmp(cp[i], diskname, sizeof(cp[i])) == 0) {
			valid = TRUE;			
			break;
		}
	}
	free(cp);

	if(!valid)
		usage();

	(void)snprintf(buf, sizeof(buf), "/dev/%s", diskname);
	if (stat(buf, &sb) == -1)
		usage();

	return diskname;
}

char *
check_arg(int argc, char *argv[])
{
	int ch;

	while ((ch = getopt(argc, argv, "nd")) != -1) {
		switch (ch) {
		case 'n':
			Fake = TRUE;
			break;
		case 'd':
			Debug = TRUE;
			break;
		case '?':
		default:
			usage();
		}
	}
	if (argc - optind > 1 || argc == optind ||
					strlen(argv[optind]) > 11)
		usage();

	return check_disk(argv[optind]);
}

int
DisplayHelp(char *file)
{
    char *fname = NULL;
    int ret = 0;
    WINDOW *w = savescr();
    
	use_helpfile(NULL);
	use_helpline(NULL);

	fname = FindHelpFile(file);
	
    if (fname) {
		dialog_textbox(file, fname, LINES, COLS);
		free(fname);
	} else {
		beep();
		ret = 1;
	}

    restorescr(w);
    return ret;
}

char *
FindHelpFile(char *file)
{
	char buf[FILENAME_MAX];

	*buf = '\0';

	if (!file)
		return NULL;

	if (file[0] == '/')
		return file;

	snprintf(buf, sizeof(buf), "%s/%s.hlp", DOCS, file);
	if (file_readable(buf))
		return strdup(buf);

	snprintf(buf, sizeof(buf), "/usr/src/usr.sbin/sysinstall/help/%s.hlp", file);
	if (file_readable(buf))
		return strdup(buf);

    snprintf(buf, sizeof(buf), "/usr/src/usr.sbin/sysinstall/help/%s.TXT", file);
    if (file_readable(buf))
		return strdup(buf);

    return NULL;
}

/* sane strncpy() function */
char *
sstrncpy(char *dst, const char *src, int size)
{
	char *ret;

	ret = strncpy(dst, src, size);
	dst[size] = '\0';
	return ret;
}

void
restorescr(WINDOW *w)
{
	touchwin(w);
	wrefresh(w);
	delwin(w);
}
