/*-
 * Copyright (c) 2006 Alex Kozlov <spam@rm-rf.kiev.ua>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer,
 *    verbatim and that no modifications are made prior to this
 *    point in the file.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY JORDAN HUBBARD ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL JORDAN HUBBARD OR HIS PETS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, LIFE OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Id: msg.c,v 1.7 2006/04/23 00:35:26 stalker Exp $
 */ 

#include "fdisk.h"

int
uni_msg(int mode, char *buf, char *fmt, ...)
{
    va_list args;
	int ret;
    WINDOW *w;
	static char str[BUFSIZ];
    static char input_buffer[256];

    w = savescr();

    va_start(args, fmt);
    vsnprintf(str, BUFSIZ, fmt, args);
    va_end(args);
	use_helpline(NULL);
	use_helpfile(NULL);

	switch(mode) {
	case MSG_FATAL:
		beep();
		dialog_clear();
		dialog_mesgbox("Fatal error", str, -1, -1);
		Shutdown(1);
	case MSG_NOTIFY:
		dialog_msgbox(NULL, str, -1, -1, 0);
		sleep(1);
		ret = -1;
		break;
	case MSG_CONFIRM:
		dialog_notify(str);
		ret = -1;
		break;
	case MSG_NOYES:
    	ret = dialog_noyes("User Confirmation Requested", str, -1, -1);
		break;
	case MSG_YESNO:
    	ret = dialog_yesno("User Confirmation Requested", str, -1, -1);
		break;
	case MSG_GETINPUT:
		if (buf) 
			sstrncpy(input_buffer, buf, sizeof(input_buffer));
		else
			input_buffer[0] = '\0';
		
		ret = dialog_inputbox("Value Required", str, -1, -1, input_buffer);
		ret = ret ? 0 : (int)input_buffer;
		break;
	default:
		ret = -1;
		break;
	}
    restorescr(w);
	return ret;
}
